'''
    Annotated CIMantic Graphs data profile init file for cim17v40
    Generated by CIMTool http://cimtool.org
'''
from cimgraph.data_profile.cimhub_ufls.cimhub_ufls import (
    PU, Accumulator, AccumulatorLimit, AccumulatorLimitSet, AccumulatorReset,
    AccumulatorValue, ACDCTerminal, ACLineSegment, ACLineSegmentPhase,
    ActivePower, ActivePowerLimit, ActivePowerPerFrequency, Analog,
    AnalogControl, AnalogLimit, AnalogLimitSet, AnalogValue, AngleDegrees,
    AngleRadians, ApparentPower, ApparentPowerLimit, Area, AreaConfiguration,
    Asset, AssetContainer, AssetInfo, AssetKind, AssetOrganisationRole,
    AsynchronousMachine, AsynchronousMachineKind, BaseFrequency, BasePower,
    BaseVoltage, BasicIntervalSchedule, BatteryStateKind, BatteryUnit, Bay,
    BranchGroup, BranchGroupTerminal, Breaker, BreakerConfiguration,
    BusbarConfiguration, BusbarSection, BusbarSectionInfo, Bushing,
    BushingInfo, BushingInsulationKind, BusNameMarker, CableConstructionKind,
    CableInfo, CableOuterJacketKind, CableShieldMaterialKind, Capacitance,
    CatalogAssetType, Clamp, Command, CompositeSwitch,
    ConcentricNeutralCableInfo, Conductance, ConductancePerLength,
    ConductingEquipment, Conductor, ConformLoad, ConformLoadGroup,
    ConformLoadSchedule, ConnectivityNode, ConnectivityNodeContainer,
    Connector, Control, ConverterControlModeKind, CoolantType,
    CoordinateSystem, Currency, CurrentFlow, CurrentLimit, Curve, CurveData,
    CurveStyle, Customer, CustomerAgreement, CustomerKind, Cut, DateInterval,
    DateTimeInterval, DayType, DecimalQuantity, DemandResponseProgram,
    DERCurveData, DERDynamics, DERFunction, DERGroupDispatch, DERGroupForecast,
    DERMonitorableParameter, DERParameterKind, DERUnitSymbol, Disconnector,
    Discrete, DiscreteValue, DispatchSchedule, Displacement, DistributionArea,
    DuctBank, EarthFaultCompensator, EndDevice, EndDeviceControl,
    EndDeviceGroup, EnergyArea, EnergyConnection, EnergyConnectionProfile,
    EnergyConsumer, EnergyConsumerPhase, EnergySource, EnergySourcePhase,
    Equipment, EquipmentContainer, ExternalNetworkInjection, Feeder,
    FeederArea, FloatQuantity, FlowDirectionKind, Frequency,
    FrequencyConverter, FrequencyProtectionFunctionBlock, FunctionBlock,
    FunctionInputVariable, FunctionOutputVariable, Fuse, GeneratingUnit,
    GeneratorControlMode, GeneratorControlSource, GenUnitOpSchedule,
    GeographicalRegion, GrossToNetActivePowerCurve, Ground, GroundDisconnector,
    GroundingImpedance, House, HouseCooling, HouseHeating,
    HouseThermalIntegrity, HydroPump, IdentifiedObject, Identity,
    IEC61970CIMVersion, IEEE1547AbnormalPerfomanceCategory,
    IEEE1547ControlSettings, IEEE1547Info, IEEE1547IslandingCategory,
    IEEE1547NormalPerformanceCategory, IEEE1547Setting, IEEE1547TripSettings,
    Impedance, IntegerQuantity, InterrupterUnitInfo, InterruptingMediumKind,
    InUseStateKind, IOPoint, IrregularIntervalSchedule, IrregularTimePoint,
    Jumper, Junction, KiloActivePower, Length, Limit, LimitSet, Line,
    LinearShuntCompensator, LinearShuntCompensatorPhase, LoadArea,
    LoadBreakSwitch, LoadGroup, LoadResponseCharacteristic, Location, Mass,
    Measurement, MeasurementValue, MeasurementValueQuality,
    MeasurementValueSource, Microgrid, Minutes, Money, MonthDay,
    MonthDayInterval, MutualCoupling, Name, NameType, NameTypeAuthority,
    NoLoadTest, NonConformLoad, NonConformLoadGroup, NonConformLoadSchedule,
    NonlinearShuntCompensator, NonlinearShuntCompensatorPhase,
    NonlinearShuntCompensatorPhasePoint, NonlinearShuntCompensatorPoint,
    OpenCircuitTest, OperatingMechanismInfo, OperatingMechanismKind,
    OperatingParticipant, OperatingShare, OperationalLimit,
    OperationalLimitDirectionKind, OperationalLimitSet, OperationalLimitType,
    OrderedPhaseCodeKind, OverfrequencyTripCurve, OverheadWireInfo,
    OvervoltageTripCurve, ParallelLineSegment, PerCent, PerLengthImpedance,
    PerLengthLineParameter, PerLengthPhaseImpedance,
    PerLengthSequenceImpedance, PetersenCoil, PetersenCoilModeKind, PhaseCode,
    PhaseImpedanceData, PhaseShuntConnectionKind, PhaseTapChanger,
    PhaseTapChangerAsymmetrical, PhaseTapChangerLinear,
    PhaseTapChangerNonLinear, PhaseTapChangerSymmetrical, PhaseTapChangerTable,
    PhaseTapChangerTablePoint, PhaseTapChangerTabular, PhotovoltaicUnit, Plant,
    PositionPoint, PowerCutZone, PowerElectronicsConnection,
    PowerElectronicsConnectionPhase, PowerElectronicsUnit,
    PowerElectronicsWindUnit, PowerSystemResource, PowerTransformer,
    PowerTransformerEnd, PowerTransformerInfo, Pressure, ProductAssetModel,
    ProtectedSwitch, ProtectionEquipment, ProtectionFunctionBlock,
    ProtectionSettingsGroup, PSRType, Quality61850, RaiseLowerCommand,
    RatioTapChanger, RatioTapChangerTable, RatioTapChangerTablePoint,
    Reactance, ReactancePerLength, ReactiveCapabilityCurve, ReactivePower,
    RealEnergy, Recloser, RegularIntervalSchedule, RegularTimePoint,
    RegulatingCondEq, RegulatingControl, RegulatingControlModeKind,
    RegulationSchedule, ReportingGroup, ReportingSuperGroup, Resistance,
    ResistancePerLength, RightOfWay, RotatingMachine, RotationSpeed,
    SchedulingArea, Season, SeasonDayTypeSchedule, SecondaryArea, Seconds,
    Sectionaliser, SeriesCompensator, SetPoint, ShortCircuitRotorKind,
    ShortCircuitTest, ShuntCompensator, ShuntCompensatorInfo,
    ShuntCompensatorPhase, SinglePhaseKind, SolarGeneratingUnit, Source,
    StaticVarCompensator, StationSupply, Status, StringMeasurement,
    StringMeasurementValue, StringQuantity, Structure, SubGeographicalRegion,
    SubLoadArea, SubSchedulingArea, Substation, Susceptance,
    SusceptancePerLength, SVCControlMode, SvShuntCompensatorSections,
    SvTapStep, Switch, SwitchArea, SwitchInfo, SwitchPhase, SwitchSchedule,
    SynchronousMachine, SynchronousMachineKind,
    SynchronousMachineOperatingMode, TapChanger, TapChangerControl,
    TapChangerInfo, TapChangerTablePoint, TapeShieldCableInfo, TapSchedule,
    Temperature, Terminal, ThermostatController, ThermostatControlMode,
    TimeInterval, TimeIntervalKind, TopologicalIsland, TopologicalNode,
    TransformerControlMode, TransformerCoreAdmittance, TransformerEnd,
    TransformerEndInfo, TransformerMeshImpedance, TransformerStarImpedance,
    TransformerTank, TransformerTankEnd, TransformerTankInfo, TransformerTest,
    UnderFrequencyProtectionFunctionBlock, UnderfrequencyTripCurve,
    UndervoltageTripCurve, UnitMultiplier, UnitSymbol, UsagePointGroup,
    Validity, ValueAliasSet, ValueToAlias, Version, Voltage,
    VoltageControlZone, VoltageLevel, VoltageLimit, VoltagePerReactivePower,
    VoltVarCurve, VoltWattCurve, Volume, WattVarCurve,
    WideAreaProtectionFunctionBlock, WindGeneratingUnit, WindGenUnitKind,
    WindingConnection, WireAssemblyInfo, WireInfo, WireInsulationKind,
    WireMaterialKind, WirePhaseInfo, WirePosition, WireSpacingInfo,
    WireUsageKind)

__all__ = [
    'Identity',
    'ACDCTerminal' ,
    'ACLineSegment' ,
    'ACLineSegmentPhase' ,
    'Accumulator' ,
    'AccumulatorLimit' ,
    'AccumulatorLimitSet' ,
    'AccumulatorReset' ,
    'AccumulatorValue' ,
    'ActivePowerLimit' ,
    'Analog' ,
    'AnalogControl' ,
    'AnalogLimit' ,
    'AnalogLimitSet' ,
    'AnalogValue' ,
    'ApparentPowerLimit' ,
    'AreaConfiguration' ,
    'Asset' ,
    'AssetContainer' ,
    'AssetInfo' ,
    'AssetKind' ,
    'AssetOrganisationRole' ,
    'AsynchronousMachine' ,
    'AsynchronousMachineKind' ,
    'BaseFrequency' ,
    'BasePower' ,
    'BaseVoltage' ,
    'BasicIntervalSchedule' ,
    'BatteryStateKind' ,
    'BatteryUnit' ,
    'Bay' ,
    'BranchGroup' ,
    'BranchGroupTerminal' ,
    'Breaker' ,
    'BreakerConfiguration' ,
    'BusNameMarker' ,
    'BusbarConfiguration' ,
    'BusbarSection' ,
    'BusbarSectionInfo' ,
    'Bushing' ,
    'BushingInfo' ,
    'BushingInsulationKind' ,
    'CableConstructionKind' ,
    'CableInfo' ,
    'CableOuterJacketKind' ,
    'CableShieldMaterialKind' ,
    'CatalogAssetType' ,
    'Clamp' ,
    'Command' ,
    'CompositeSwitch' ,
    'ConcentricNeutralCableInfo' ,
    'ConductingEquipment' ,
    'Conductor' ,
    'ConformLoad' ,
    'ConformLoadGroup' ,
    'ConformLoadSchedule' ,
    'ConnectivityNode' ,
    'ConnectivityNodeContainer' ,
    'Connector' ,
    'Control' ,
    'ConverterControlModeKind' ,
    'CoolantType' ,
    'CoordinateSystem' ,
    'Currency' ,
    'CurrentLimit' ,
    'Curve' ,
    'CurveData' ,
    'CurveStyle' ,
    'Customer' ,
    'CustomerAgreement' ,
    'CustomerKind' ,
    'Cut' ,
    'DERCurveData' ,
    'DERDynamics' ,
    'DERFunction' ,
    'DERGroupDispatch' ,
    'DERGroupForecast' ,
    'DERMonitorableParameter' ,
    'DERParameterKind' ,
    'DERUnitSymbol' ,
    'DateInterval' ,
    'DateTimeInterval' ,
    'DayType' ,
    'DecimalQuantity' ,
    'DemandResponseProgram' ,
    'Disconnector' ,
    'Discrete' ,
    'DiscreteValue' ,
    'DispatchSchedule' ,
    'DistributionArea' ,
    'DuctBank' ,
    'EarthFaultCompensator' ,
    'EndDevice' ,
    'EndDeviceControl' ,
    'EndDeviceGroup' ,
    'EnergyArea' ,
    'EnergyConnection' ,
    'EnergyConnectionProfile' ,
    'EnergyConsumer' ,
    'EnergyConsumerPhase' ,
    'EnergySource' ,
    'EnergySourcePhase' ,
    'Equipment' ,
    'EquipmentContainer' ,
    'ExternalNetworkInjection' ,
    'Feeder' ,
    'FeederArea' ,
    'FloatQuantity' ,
    'FlowDirectionKind' ,
    'FrequencyConverter' ,
    'FrequencyProtectionFunctionBlock' ,
    'FunctionBlock' ,
    'FunctionInputVariable' ,
    'FunctionOutputVariable' ,
    'Fuse' ,
    'GenUnitOpSchedule' ,
    'GeneratingUnit' ,
    'GeneratorControlMode' ,
    'GeneratorControlSource' ,
    'GeographicalRegion' ,
    'GrossToNetActivePowerCurve' ,
    'Ground' ,
    'GroundDisconnector' ,
    'GroundingImpedance' ,
    'House' ,
    'HouseCooling' ,
    'HouseHeating' ,
    'HouseThermalIntegrity' ,
    'HydroPump' ,
    'IEC61970CIMVersion' ,
    'IEEE1547AbnormalPerfomanceCategory' ,
    'IEEE1547ControlSettings' ,
    'IEEE1547Info' ,
    'IEEE1547IslandingCategory' ,
    'IEEE1547NormalPerformanceCategory' ,
    'IEEE1547Setting' ,
    'IEEE1547TripSettings' ,
    'IOPoint' ,
    'IdentifiedObject' ,
    'InUseStateKind' ,
    'IntegerQuantity' ,
    'InterrupterUnitInfo' ,
    'InterruptingMediumKind' ,
    'IrregularIntervalSchedule' ,
    'IrregularTimePoint' ,
    'Jumper' ,
    'Junction' ,
    'Limit' ,
    'LimitSet' ,
    'Line' ,
    'LinearShuntCompensator' ,
    'LinearShuntCompensatorPhase' ,
    'LoadArea' ,
    'LoadBreakSwitch' ,
    'LoadGroup' ,
    'LoadResponseCharacteristic' ,
    'Location' ,
    'Measurement' ,
    'MeasurementValue' ,
    'MeasurementValueQuality' ,
    'MeasurementValueSource' ,
    'Microgrid' ,
    'MonthDayInterval' ,
    'MutualCoupling' ,
    'Name' ,
    'NameType' ,
    'NameTypeAuthority' ,
    'NoLoadTest' ,
    'NonConformLoad' ,
    'NonConformLoadGroup' ,
    'NonConformLoadSchedule' ,
    'NonlinearShuntCompensator' ,
    'NonlinearShuntCompensatorPhase' ,
    'NonlinearShuntCompensatorPhasePoint' ,
    'NonlinearShuntCompensatorPoint' ,
    'OpenCircuitTest' ,
    'OperatingMechanismInfo' ,
    'OperatingMechanismKind' ,
    'OperatingParticipant' ,
    'OperatingShare' ,
    'OperationalLimit' ,
    'OperationalLimitDirectionKind' ,
    'OperationalLimitSet' ,
    'OperationalLimitType' ,
    'OrderedPhaseCodeKind' ,
    'OverfrequencyTripCurve' ,
    'OverheadWireInfo' ,
    'OvervoltageTripCurve' ,
    'PSRType' ,
    'ParallelLineSegment' ,
    'PerLengthImpedance' ,
    'PerLengthLineParameter' ,
    'PerLengthPhaseImpedance' ,
    'PerLengthSequenceImpedance' ,
    'PetersenCoil' ,
    'PetersenCoilModeKind' ,
    'PhaseCode' ,
    'PhaseImpedanceData' ,
    'PhaseShuntConnectionKind' ,
    'PhaseTapChanger' ,
    'PhaseTapChangerAsymmetrical' ,
    'PhaseTapChangerLinear' ,
    'PhaseTapChangerNonLinear' ,
    'PhaseTapChangerSymmetrical' ,
    'PhaseTapChangerTable' ,
    'PhaseTapChangerTablePoint' ,
    'PhaseTapChangerTabular' ,
    'PhotovoltaicUnit' ,
    'Plant' ,
    'PositionPoint' ,
    'PowerCutZone' ,
    'PowerElectronicsConnection' ,
    'PowerElectronicsConnectionPhase' ,
    'PowerElectronicsUnit' ,
    'PowerElectronicsWindUnit' ,
    'PowerSystemResource' ,
    'PowerTransformer' ,
    'PowerTransformerEnd' ,
    'PowerTransformerInfo' ,
    'ProductAssetModel' ,
    'ProtectedSwitch' ,
    'ProtectionEquipment' ,
    'ProtectionFunctionBlock' ,
    'ProtectionSettingsGroup' ,
    'Quality61850' ,
    'RaiseLowerCommand' ,
    'RatioTapChanger' ,
    'RatioTapChangerTable' ,
    'RatioTapChangerTablePoint' ,
    'ReactiveCapabilityCurve' ,
    'Recloser' ,
    'RegularIntervalSchedule' ,
    'RegularTimePoint' ,
    'RegulatingCondEq' ,
    'RegulatingControl' ,
    'RegulatingControlModeKind' ,
    'RegulationSchedule' ,
    'ReportingGroup' ,
    'ReportingSuperGroup' ,
    'RightOfWay' ,
    'RotatingMachine' ,
    'SVCControlMode' ,
    'SchedulingArea' ,
    'Season' ,
    'SeasonDayTypeSchedule' ,
    'SecondaryArea' ,
    'Sectionaliser' ,
    'SeriesCompensator' ,
    'SetPoint' ,
    'ShortCircuitRotorKind' ,
    'ShortCircuitTest' ,
    'ShuntCompensator' ,
    'ShuntCompensatorInfo' ,
    'ShuntCompensatorPhase' ,
    'SinglePhaseKind' ,
    'SolarGeneratingUnit' ,
    'StaticVarCompensator' ,
    'StationSupply' ,
    'Status' ,
    'StringMeasurement' ,
    'StringMeasurementValue' ,
    'StringQuantity' ,
    'Structure' ,
    'SubGeographicalRegion' ,
    'SubLoadArea' ,
    'SubSchedulingArea' ,
    'Substation' ,
    'SvShuntCompensatorSections' ,
    'SvTapStep' ,
    'Switch' ,
    'SwitchArea' ,
    'SwitchInfo' ,
    'SwitchPhase' ,
    'SwitchSchedule' ,
    'SynchronousMachine' ,
    'SynchronousMachineKind' ,
    'SynchronousMachineOperatingMode' ,
    'TapChanger' ,
    'TapChangerControl' ,
    'TapChangerInfo' ,
    'TapChangerTablePoint' ,
    'TapSchedule' ,
    'TapeShieldCableInfo' ,
    'Terminal' ,
    'ThermostatControlMode' ,
    'ThermostatController' ,
    'TimeInterval' ,
    'TimeIntervalKind' ,
    'TopologicalIsland' ,
    'TopologicalNode' ,
    'TransformerControlMode' ,
    'TransformerCoreAdmittance' ,
    'TransformerEnd' ,
    'TransformerEndInfo' ,
    'TransformerMeshImpedance' ,
    'TransformerStarImpedance' ,
    'TransformerTank' ,
    'TransformerTankEnd' ,
    'TransformerTankInfo' ,
    'TransformerTest' ,
    'UnderFrequencyProtectionFunctionBlock' ,
    'UnderfrequencyTripCurve' ,
    'UndervoltageTripCurve' ,
    'UnitMultiplier' ,
    'UnitSymbol' ,
    'UsagePointGroup' ,
    'Validity' ,
    'ValueAliasSet' ,
    'ValueToAlias' ,
    'Version' ,
    'VoltVarCurve' ,
    'VoltWattCurve' ,
    'VoltageControlZone' ,
    'VoltageLevel' ,
    'VoltageLimit' ,
    'WattVarCurve' ,
    'WideAreaProtectionFunctionBlock' ,
    'WindGenUnitKind' ,
    'WindGeneratingUnit' ,
    'WindingConnection' ,
    'WireAssemblyInfo' ,
    'WireInfo' ,
    'WireInsulationKind' ,
    'WireMaterialKind' ,
    'WirePhaseInfo' ,
    'WirePosition' ,
    'WireSpacingInfo' ,
    'WireUsageKind' ,
    'MonthDay' ,
    'Susceptance' ,
    'ReactivePower' ,
    'Pressure' ,
    'Capacitance' ,
    'Temperature' ,
    'Seconds' ,
    'Voltage' ,
    'ActivePowerPerFrequency' ,
    'SusceptancePerLength' ,
    'Frequency' ,
    'PU' ,
    'VoltagePerReactivePower' ,
    'ActivePower' ,
    'Mass' ,
    'ConductancePerLength' ,
    'Reactance' ,
    'Minutes' ,
    'AngleDegrees' ,
    'RotationSpeed' ,
    'RealEnergy' ,
    'Conductance' ,
    'ReactancePerLength' ,
    'Resistance' ,
    'Money' ,
    'Area' ,
    'Volume' ,
    'Length' ,
    'KiloActivePower' ,
    'ResistancePerLength' ,
    'Displacement' ,
    'ApparentPower' ,
    'PerCent' ,
    'AngleRadians' ,
    'CurrentFlow' ,
    'Source' ,
    'Impedance'
]
