'''
    Annotated CIMantic Graphs data profile init file for cim17v40
    Generated by CIMTool http://cimtool.org
'''
from cimgraph.data_profile.cim17v40.cim17v40 import (
    PU, Accumulator, AccumulatorLimit, AccumulatorLimitSet, AccumulatorReset,
    AccumulatorValue, ACDCConverter, ACDCConverterDCTerminal, ACDCTerminal,
    ACLineSegment, ACLineSegmentPhase, ActivePower, ActivePowerChangeRate,
    ActivePowerLimit, ActivePowerPerCurrentFlow, ActivePowerPerFrequency,
    Analog, AnalogControl, AnalogLimit, AnalogLimitSet, AnalogValue,
    AngleDegrees, AngleRadians, ApparentPower, ApparentPowerLimit, Asset,
    AssetContainer, AssetInfo, AssetKind, AsynchronousMachine,
    AsynchronousMachineDynamics, AsynchronousMachineKind, BaseFrequency,
    BasePower, BaseVoltage, BasicIntervalSchedule, BatteryStateKind,
    BatteryUnit, Bay, BranchGroup, BranchGroupTerminal, Breaker,
    BreakerConfiguration, BusbarConfiguration, BusbarSection,
    BusbarSectionInfo, Bushing, BushingInfo, BushingInsulationKind,
    BusNameMarker, CableConstructionKind, CableInfo, CableOuterJacketKind,
    CableShieldMaterialKind, CalculationMethodHierarchy, Capacitance,
    CapacitancePerLength, CatalogAssetType, ChangeSetMember, Circuit, Clamp,
    ClampAction, Classification, Command, CompositeSwitch,
    ConcentricNeutralCableInfo, Conductance, ConductancePerLength,
    ConductingEquipment, Conductor, ConformLoad, ConformLoadGroup,
    ConformLoadSchedule, ConnectivityNode, ConnectivityNodeContainer,
    Connector, Control, ControlAction, ControlArea, ControlAreaGeneratingUnit,
    CoolantType, CoordinateSystem, CSCDynamics, CsConverter,
    CsOperatingModeKind, CsPpccControlKind, Currency, CurrentFlow,
    CurrentLimit, Curve, CurveData, CurveStyle, Customer, CustomerAgreement,
    CustomerKind, Cut, CutAction, DateInterval, DateTimeInterval, DayType,
    DCBaseTerminal, DCBreaker, DCBusbar, DCChopper, DCConductingEquipment,
    DCConverterOperatingModeKind, DCConverterUnit, DCDisconnector,
    DCEquipmentContainer, DCGround, DCLine, DCLineSegment, DCNode,
    DCPolarityKind, DCSeriesDevice, DCShunt, DCSwitch, DCTerminal,
    DCTopologicalIsland, DCTopologicalNode, DecimalQuantity,
    DemandResponseProgram, DERCurveData, DERFunction, DERGroupDispatch,
    DERGroupForecast, DERMonitorableParameter, DERParameterKind, DERUnitSymbol,
    DisconnectingCircuitBreaker, Disconnector, Discrete, DiscreteValue,
    DispatchSchedule, Displacement, DuctBank, EarthFaultCompensator, EndDevice,
    EndDeviceControl, EndDeviceGroup, EnergyArea, EnergyConnection,
    EnergyConsumer, EnergyConsumerAction, EnergyConsumerPhase,
    EnergySchedulingType, EnergySource, EnergySourceAction, EnergySourcePhase,
    Equipment, EquipmentContainer, ExternalNetworkInjection, Feeder,
    FloatQuantity, FlowDirectionKind, Frequency, FrequencyConverter, Fuse,
    GeneratingUnit, GeneratorControlMode, GeneratorControlSource,
    GenUnitOpSchedule, GeographicalRegion, Ground, GroundAction,
    GroundDisconnector, GroundingImpedance, Hours, HydroPump, IdentifiedObject,
    Identity, IEC61970CIMVersion, Impedance, Inductance, InductancePerLength,
    InstanceSet, IntegerQuantity, InterrupterUnitInfo, InterruptingMediumKind,
    InUseStateKind, IOPoint, IOPointSource, IrregularIntervalSchedule,
    IrregularTimePoint, Jumper, JumperAction, Junction, KiloActivePower,
    Length, Limit, LimitSet, Line, LinearShuntCompensator,
    LinearShuntCompensatorPhase, LoadArea, LoadBreakSwitch, LoadDynamics,
    LoadGroup, LoadResponseCharacteristic, Location, Mass, Measurement,
    MeasurementAction, MeasurementValue, MeasurementValueQuality,
    MeasurementValueSource, MeterReadSchedule, Minutes, Money, MonthDay,
    MonthDayInterval, MutualCoupling, Name, NameType, NameTypeAuthority,
    NoLoadTest, NonConformLoad, NonConformLoadGroup, NonConformLoadSchedule,
    NonlinearShuntCompensator, NonlinearShuntCompensatorPhase,
    NonlinearShuntCompensatorPhasePoint, NonlinearShuntCompensatorPoint,
    OldPerson, OpenCircuitTest, OperatingMechanismInfo, OperatingMechanismKind,
    OperatingParticipant, OperatingShare, OperationalLimit,
    OperationalLimitDirectionKind, OperationalLimitSet, OperationalLimitType,
    OperationalRestriction, OperatonalLimitTypeScaling, Outage,
    OverheadWireInfo, PerCent, PerLengthDCLineParameter, PerLengthImpedance,
    PerLengthLineParameter, PerLengthPhaseImpedance,
    PerLengthSequenceImpedance, PetersenCoil, PetersenCoilModeKind, PhaseCode,
    PhaseImpedanceData, PhaseShuntConnectionKind, PhaseTapChanger,
    PhaseTapChangerAsymmetrical, PhaseTapChangerLinear,
    PhaseTapChangerNonLinear, PhaseTapChangerSymmetrical, PhaseTapChangerTable,
    PhaseTapChangerTablePoint, PhaseTapChangerTabular, PhotoVoltaicUnit, Plant,
    PositionPoint, PowerCutZone, PowerElectronicsConnection,
    PowerElectronicsConnectionPhase, PowerElectronicsUnit,
    PowerElectronicsWindUnit, PowerSystemResource, PowerTransformer,
    PowerTransformerEnd, PowerTransformerInfo, Pressure, ProductAssetModel,
    ProtectedSwitch, ProtectiveActionAdjustment, PSRType, Quality61850,
    RaiseLowerCommand, RatioTapChanger, RatioTapChangerTable,
    RatioTapChangerTablePoint, Reactance, ReactancePerLength,
    ReactiveCapabilityCurve, ReactivePower, RealEnergy, Recloser,
    RegularIntervalSchedule, RegularTimePoint, RegulatingCondEq,
    RegulatingControl, RegulatingControlModeKind, RegulationBranchKind,
    RegulationSchedule, RemoteControl, RemoteSource, ReportingGroup,
    ReportingSuperGroup, Resistance, ResistancePerLength, RotatingMachine,
    RotationSpeed, Season, SeasonDayTypeSchedule, Seconds, Sectionaliser,
    SeriesCompensator, SetPoint, ShortCircuitRotorKind, ShortCircuitTest,
    ShuntCompensator, ShuntCompensatorAction, ShuntCompensatorControl,
    ShuntCompensatorInfo, ShuntCompensatorPhase, ShuntImpedanceControlKind,
    ShuntImpedanceLocalControlKind, SinglePhaseKind, Source,
    StaticVarCompensator, StaticVarCompensatorDynamics, StationSupply, Status,
    StringMeasurement, StringMeasurementValue, StringQuantity,
    SubGeographicalRegion, SubLoadArea, Substation, Susceptance,
    SusceptancePerLength, SVCControlMode, SvShuntCompensatorSections,
    SvTapStep, Switch, SwitchAction, SwitchActionKind, SwitchInfo, SwitchPhase,
    SwitchSchedule, SynchronousMachine, SynchronousMachineDynamics,
    SynchronousMachineKind, SynchronousMachineOperatingMode, TapChanger,
    TapChangerControl, TapChangerInfo, TapChangerTablePoint,
    TapeShieldCableInfo, TapSchedule, TempEquipActionKind, Temperature,
    Terminal, TimeInterval, TimeIntervalKind, TopologicalIsland,
    TopologicalNode, TransformerControlMode, TransformerCoreAdmittance,
    TransformerEnd, TransformerEndInfo, TransformerMeshImpedance,
    TransformerObservation, TransformerStarImpedance, TransformerTank,
    TransformerTankEnd, TransformerTankInfo, TransformerTest, UnitMultiplier,
    UnitSymbol, UsagePointGroup, Validity, ValueAliasSet, ValueToAlias,
    Version, Voltage, VoltageControlZone, VoltageLevel, VoltageLimit,
    VoltagePerReactivePower, Volume, VsCapabilityCurve, VSCDynamics,
    VsConverter, VsPpccControlKind, VsQpccControlKind, WindGeneratingUnit,
    WindGenUnitKind, WindingConnection, WindingInsulation,
    WindTurbineType3or4Dynamics, WireAssemblyInfo, WireInfo,
    WireInsulationKind, WireMaterialKind, WirePhaseInfo, WirePosition,
    WireSegment, WireSegmentPhase, WireSpacingInfo, WireUsageKind)

__all__ = [
    'Identity',
    'ACDCConverter' ,
    'ACDCConverterDCTerminal' ,
    'ACDCTerminal' ,
    'ACLineSegment' ,
    'ACLineSegmentPhase' ,
    'Accumulator' ,
    'AccumulatorLimit' ,
    'AccumulatorLimitSet' ,
    'AccumulatorReset' ,
    'AccumulatorValue' ,
    'ActivePowerLimit' ,
    'Analog' ,
    'AnalogControl' ,
    'AnalogLimit' ,
    'AnalogLimitSet' ,
    'AnalogValue' ,
    'ApparentPowerLimit' ,
    'Asset' ,
    'AssetContainer' ,
    'AssetInfo' ,
    'AssetKind' ,
    'AsynchronousMachine' ,
    'AsynchronousMachineDynamics' ,
    'AsynchronousMachineKind' ,
    'BaseFrequency' ,
    'BasePower' ,
    'BaseVoltage' ,
    'BasicIntervalSchedule' ,
    'BatteryStateKind' ,
    'BatteryUnit' ,
    'Bay' ,
    'BranchGroup' ,
    'BranchGroupTerminal' ,
    'Breaker' ,
    'BreakerConfiguration' ,
    'BusNameMarker' ,
    'BusbarConfiguration' ,
    'BusbarSection' ,
    'BusbarSectionInfo' ,
    'Bushing' ,
    'BushingInfo' ,
    'BushingInsulationKind' ,
    'CSCDynamics' ,
    'CableConstructionKind' ,
    'CableInfo' ,
    'CableOuterJacketKind' ,
    'CableShieldMaterialKind' ,
    'CalculationMethodHierarchy' ,
    'CatalogAssetType' ,
    'ChangeSetMember' ,
    'Circuit' ,
    'Clamp' ,
    'ClampAction' ,
    'Command' ,
    'CompositeSwitch' ,
    'ConcentricNeutralCableInfo' ,
    'ConductingEquipment' ,
    'Conductor' ,
    'ConformLoad' ,
    'ConformLoadGroup' ,
    'ConformLoadSchedule' ,
    'ConnectivityNode' ,
    'ConnectivityNodeContainer' ,
    'Connector' ,
    'Control' ,
    'ControlAction' ,
    'ControlArea' ,
    'ControlAreaGeneratingUnit' ,
    'CoolantType' ,
    'CoordinateSystem' ,
    'CsConverter' ,
    'CsOperatingModeKind' ,
    'CsPpccControlKind' ,
    'Currency' ,
    'CurrentLimit' ,
    'Curve' ,
    'CurveData' ,
    'CurveStyle' ,
    'Customer' ,
    'CustomerAgreement' ,
    'CustomerKind' ,
    'Cut' ,
    'CutAction' ,
    'DCBaseTerminal' ,
    'DCBreaker' ,
    'DCBusbar' ,
    'DCChopper' ,
    'DCConductingEquipment' ,
    'DCConverterOperatingModeKind' ,
    'DCConverterUnit' ,
    'DCDisconnector' ,
    'DCEquipmentContainer' ,
    'DCGround' ,
    'DCLine' ,
    'DCLineSegment' ,
    'DCNode' ,
    'DCPolarityKind' ,
    'DCSeriesDevice' ,
    'DCShunt' ,
    'DCSwitch' ,
    'DCTerminal' ,
    'DCTopologicalIsland' ,
    'DCTopologicalNode' ,
    'DERCurveData' ,
    'DERFunction' ,
    'DERGroupDispatch' ,
    'DERGroupForecast' ,
    'DERMonitorableParameter' ,
    'DERParameterKind' ,
    'DERUnitSymbol' ,
    'DateInterval' ,
    'DateTimeInterval' ,
    'DayType' ,
    'DecimalQuantity' ,
    'DemandResponseProgram' ,
    'DisconnectingCircuitBreaker' ,
    'Disconnector' ,
    'Discrete' ,
    'DiscreteValue' ,
    'DispatchSchedule' ,
    'DuctBank' ,
    'EarthFaultCompensator' ,
    'EndDevice' ,
    'EndDeviceControl' ,
    'EndDeviceGroup' ,
    'EnergyArea' ,
    'EnergyConnection' ,
    'EnergyConsumer' ,
    'EnergyConsumerAction' ,
    'EnergyConsumerPhase' ,
    'EnergySchedulingType' ,
    'EnergySource' ,
    'EnergySourceAction' ,
    'EnergySourcePhase' ,
    'Equipment' ,
    'EquipmentContainer' ,
    'ExternalNetworkInjection' ,
    'Feeder' ,
    'FloatQuantity' ,
    'FlowDirectionKind' ,
    'FrequencyConverter' ,
    'Fuse' ,
    'GenUnitOpSchedule' ,
    'GeneratingUnit' ,
    'GeneratorControlMode' ,
    'GeneratorControlSource' ,
    'GeographicalRegion' ,
    'Ground' ,
    'GroundAction' ,
    'GroundDisconnector' ,
    'GroundingImpedance' ,
    'HydroPump' ,
    'IEC61970CIMVersion' ,
    'IOPoint' ,
    'IOPointSource' ,
    'IdentifiedObject' ,
    'InUseStateKind' ,
    'InstanceSet' ,
    'IntegerQuantity' ,
    'InterrupterUnitInfo' ,
    'InterruptingMediumKind' ,
    'IrregularIntervalSchedule' ,
    'IrregularTimePoint' ,
    'Jumper' ,
    'JumperAction' ,
    'Junction' ,
    'Limit' ,
    'LimitSet' ,
    'Line' ,
    'LinearShuntCompensator' ,
    'LinearShuntCompensatorPhase' ,
    'LoadArea' ,
    'LoadBreakSwitch' ,
    'LoadDynamics' ,
    'LoadGroup' ,
    'LoadResponseCharacteristic' ,
    'Location' ,
    'Measurement' ,
    'MeasurementAction' ,
    'MeasurementValue' ,
    'MeasurementValueQuality' ,
    'MeasurementValueSource' ,
    'MeterReadSchedule' ,
    'MonthDayInterval' ,
    'MutualCoupling' ,
    'Name' ,
    'NameType' ,
    'NameTypeAuthority' ,
    'NoLoadTest' ,
    'NonConformLoad' ,
    'NonConformLoadGroup' ,
    'NonConformLoadSchedule' ,
    'NonlinearShuntCompensator' ,
    'NonlinearShuntCompensatorPhase' ,
    'NonlinearShuntCompensatorPhasePoint' ,
    'NonlinearShuntCompensatorPoint' ,
    'OldPerson' ,
    'OpenCircuitTest' ,
    'OperatingMechanismInfo' ,
    'OperatingMechanismKind' ,
    'OperatingParticipant' ,
    'OperatingShare' ,
    'OperationalLimit' ,
    'OperationalLimitDirectionKind' ,
    'OperationalLimitSet' ,
    'OperationalLimitType' ,
    'OperationalRestriction' ,
    'OperatonalLimitTypeScaling' ,
    'Outage' ,
    'OverheadWireInfo' ,
    'PSRType' ,
    'PerLengthDCLineParameter' ,
    'PerLengthImpedance' ,
    'PerLengthLineParameter' ,
    'PerLengthPhaseImpedance' ,
    'PerLengthSequenceImpedance' ,
    'PetersenCoil' ,
    'PetersenCoilModeKind' ,
    'PhaseCode' ,
    'PhaseImpedanceData' ,
    'PhaseShuntConnectionKind' ,
    'PhaseTapChanger' ,
    'PhaseTapChangerAsymmetrical' ,
    'PhaseTapChangerLinear' ,
    'PhaseTapChangerNonLinear' ,
    'PhaseTapChangerSymmetrical' ,
    'PhaseTapChangerTable' ,
    'PhaseTapChangerTablePoint' ,
    'PhaseTapChangerTabular' ,
    'PhotoVoltaicUnit' ,
    'Plant' ,
    'PositionPoint' ,
    'PowerCutZone' ,
    'PowerElectronicsConnection' ,
    'PowerElectronicsConnectionPhase' ,
    'PowerElectronicsUnit' ,
    'PowerElectronicsWindUnit' ,
    'PowerSystemResource' ,
    'PowerTransformer' ,
    'PowerTransformerEnd' ,
    'PowerTransformerInfo' ,
    'ProductAssetModel' ,
    'ProtectedSwitch' ,
    'ProtectiveActionAdjustment' ,
    'Quality61850' ,
    'RaiseLowerCommand' ,
    'RatioTapChanger' ,
    'RatioTapChangerTable' ,
    'RatioTapChangerTablePoint' ,
    'ReactiveCapabilityCurve' ,
    'Recloser' ,
    'RegularIntervalSchedule' ,
    'RegularTimePoint' ,
    'RegulatingCondEq' ,
    'RegulatingControl' ,
    'RegulatingControlModeKind' ,
    'RegulationBranchKind' ,
    'RegulationSchedule' ,
    'RemoteControl' ,
    'RemoteSource' ,
    'ReportingGroup' ,
    'ReportingSuperGroup' ,
    'RotatingMachine' ,
    'SVCControlMode' ,
    'Season' ,
    'SeasonDayTypeSchedule' ,
    'Sectionaliser' ,
    'SeriesCompensator' ,
    'SetPoint' ,
    'ShortCircuitRotorKind' ,
    'ShortCircuitTest' ,
    'ShuntCompensator' ,
    'ShuntCompensatorAction' ,
    'ShuntCompensatorControl' ,
    'ShuntCompensatorInfo' ,
    'ShuntCompensatorPhase' ,
    'ShuntImpedanceControlKind' ,
    'ShuntImpedanceLocalControlKind' ,
    'SinglePhaseKind' ,
    'Source' ,
    'StaticVarCompensator' ,
    'StaticVarCompensatorDynamics' ,
    'StationSupply' ,
    'Status' ,
    'StringMeasurement' ,
    'StringMeasurementValue' ,
    'StringQuantity' ,
    'SubGeographicalRegion' ,
    'SubLoadArea' ,
    'Substation' ,
    'SvShuntCompensatorSections' ,
    'SvTapStep' ,
    'Switch' ,
    'SwitchAction' ,
    'SwitchActionKind' ,
    'SwitchInfo' ,
    'SwitchPhase' ,
    'SwitchSchedule' ,
    'SynchronousMachine' ,
    'SynchronousMachineDynamics' ,
    'SynchronousMachineKind' ,
    'SynchronousMachineOperatingMode' ,
    'TapChanger' ,
    'TapChangerControl' ,
    'TapChangerInfo' ,
    'TapChangerTablePoint' ,
    'TapSchedule' ,
    'TapeShieldCableInfo' ,
    'TempEquipActionKind' ,
    'Terminal' ,
    'TimeInterval' ,
    'TimeIntervalKind' ,
    'TopologicalIsland' ,
    'TopologicalNode' ,
    'TransformerControlMode' ,
    'TransformerCoreAdmittance' ,
    'TransformerEnd' ,
    'TransformerEndInfo' ,
    'TransformerMeshImpedance' ,
    'TransformerObservation' ,
    'TransformerStarImpedance' ,
    'TransformerTank' ,
    'TransformerTankEnd' ,
    'TransformerTankInfo' ,
    'TransformerTest' ,
    'UnitMultiplier' ,
    'UnitSymbol' ,
    'UsagePointGroup' ,
    'VSCDynamics' ,
    'Validity' ,
    'ValueAliasSet' ,
    'ValueToAlias' ,
    'Version' ,
    'VoltageControlZone' ,
    'VoltageLevel' ,
    'VoltageLimit' ,
    'VsCapabilityCurve' ,
    'VsConverter' ,
    'VsPpccControlKind' ,
    'VsQpccControlKind' ,
    'WindGenUnitKind' ,
    'WindGeneratingUnit' ,
    'WindTurbineType3or4Dynamics' ,
    'WindingConnection' ,
    'WindingInsulation' ,
    'WireAssemblyInfo' ,
    'WireInfo' ,
    'WireInsulationKind' ,
    'WireMaterialKind' ,
    'WirePhaseInfo' ,
    'WirePosition' ,
    'WireSegment' ,
    'WireSegmentPhase' ,
    'WireSpacingInfo' ,
    'WireUsageKind' ,
    'MonthDay' ,
    'ReactivePower' ,
    'Susceptance' ,
    'Pressure' ,
    'Hours' ,
    'Capacitance' ,
    'Temperature' ,
    'Seconds' ,
    'Voltage' ,
    'ActivePowerPerFrequency' ,
    'Inductance' ,
    'CapacitancePerLength' ,
    'SusceptancePerLength' ,
    'Frequency' ,
    'PU' ,
    'VoltagePerReactivePower' ,
    'ActivePower' ,
    'Mass' ,
    'ConductancePerLength' ,
    'Reactance' ,
    'Minutes' ,
    'AngleDegrees' ,
    'ActivePowerChangeRate' ,
    'RotationSpeed' ,
    'RealEnergy' ,
    'Conductance' ,
    'ReactancePerLength' ,
    'Resistance' ,
    'Money' ,
    'Classification' ,
    'ActivePowerPerCurrentFlow' ,
    'Volume' ,
    'Length' ,
    'KiloActivePower' ,
    'ResistancePerLength' ,
    'InductancePerLength' ,
    'Displacement' ,
    'ApparentPower' ,
    'PerCent' ,
    'AngleRadians' ,
    'CurrentFlow' ,
    'Impedance'
]
