'''
    Annotated CIMantic Graphs data profile init file for Profile
    Generated by CIMTool http://cimtool.org
'''
from cimgraph.data_profile.rc4_2021.rc4_2021 import (
    PU, AccountNotification, AccumulationKind, Accumulator, AccumulatorLimit,
    AccumulatorLimitSet, AccumulatorReset, AccumulatorValue, ACDCTerminal,
    ACLineSegment, ACLineSegmentPhase, ActivePower, ActivePowerChangeRate,
    ActivePowerLimit, ActivePowerPerFrequency, ActivityRecord, AggregateKind,
    AggregateScore, AirCompressor, AlertTypeList, AmiBillingReadyKind, Analog,
    AnalogControl, AnalogLimit, AnalogLimitSet, AnalogValue, Analytic,
    AnalyticKind, AnalyticScore, AnchorKind, AngleDegrees, AngleRadians,
    ApparentPower, ApparentPowerLimit, Appointment, Approver, Area, Asset,
    AssetContainer, AssetDeployment, AssetFunction, AssetGroup, AssetGroupKind,
    AssetHazardKind, AssetHealthEvent, AssetInfo, AssetKind,
    AssetLocationHazard, AssetModelUsageKind, AssetOrganisationRole,
    AssetOwner, AsynchronousMachine, AsynchronousMachineKind, Author,
    BaseFrequency, BasePower, BaseVoltage, BasicIntervalSchedule, BatteryState,
    BatteryUnit, Bay, BranchGroup, BranchGroupTerminal, Breaker,
    BreakerApplicationKind, BreakerConfiguration, BusbarConfiguration,
    BusbarSection, BusbarSectionInfo, Bushing, BushingInfo,
    BushingInsulationKind, BusNameMarker, CableConstructionKind, CableInfo,
    CableOuterJacketKind, CableShieldMaterialKind, CAESPlant,
    CalculationIntervalUnitKind, Capacitance, CatalogAssetType, Channel, Clamp,
    Class1, ClassificationCondition, CogenerationPlant, CombinedCyclePlant,
    Command, CommodityKind, CompositeSwitch, ConcentricNeutralCableInfo,
    Conductance, ConductancePerLength, ConductingEquipment, Conductor,
    ConfigurationEvent, ConformLoad, ConformLoadGroup, ConformLoadSchedule,
    ConnectivityNode, ConnectivityNodeContainer, Connector, Control,
    CoolantType, CoordinateSystem, CorporateStandardKind, CostPerEnergyUnit,
    CostPerHeatUnit, CostPerVolume, CostRate, Crew, CrewMember, CrewStatusKind,
    CrewType, Currency, CurrentFlow, CurrentLimit, Curve, CurveData,
    CurveStyle, Customer, CustomerAccount, CustomerAgreement, CustomerKind,
    CustomerNotification, Cut, DateInterval, DateTimeInterval, DayType,
    DecimalQuantity, DemandResponseProgram, DeploymentDate,
    DeploymentStateKind, DERCurveData, DERFunction, DERGroupDispatch,
    DERGroupForecast, DERMonitorableParameter, DERParameterKind, DERUnitSymbol,
    Disconnector, Discrete, DiscreteCommand, DiscreteValue,
    DispatchablePowerCapability, DispatchSchedule, Displacement, Document,
    DuctBank, EarthFaultCompensator, Editor, ElectronicAddress, Emission,
    EmissionAccount, EmissionCurve, EmissionType, EmissionValueSource,
    EndDevice, EndDeviceAction, EndDeviceCapability, EndDeviceControl,
    EndDeviceControlType, EndDeviceEvent, EndDeviceEventDetail,
    EndDeviceEventType, EndDeviceFunction, EndDeviceGroup, EndDeviceInfo,
    EndDeviceTiming, EnergyArea, EnergyConnection, EnergyConsumer,
    EnergyConsumerPhase, EnergySource, EnergySourcePhase, EnvironmentalAlert,
    EnvironmentalAnalog, EnvironmentalDataAuthority, EnvironmentalDataProvider,
    EnvironmentalInformation, EnvironmentalLocationType,
    EnvironmentalPhenomenon, EnvironmentalStringMeasurement, Equipment,
    EquipmentContainer, EquipmentFault, Estimate, ExternalNetworkInjection,
    FacilityKind, Fault, FaultCauseType, FaultImpedance, Feeder,
    FieldDispatchHistory, FieldDispatchStep, FinancialInfo, FloatQuantity,
    FlowDirectionKind, FossilFuel, Frequency, FrequencyConverter,
    FuelAllocationSchedule, FuelType, Fuse, GeneratingUnit,
    GeneratorControlMode, GeneratorControlSource, GenUnitOpCostCurve,
    GenUnitOpSchedule, GeographicalRegion, GrossToNetActivePowerCurve, Ground,
    GroundDisconnector, GroundingImpedance, HeatInputCurve, HeatRate,
    HeatRateCurve, Hours, House, HouseCooling, HouseHeating,
    HouseThermalIntegrity, HydroEnergyConversionKind,
    HydroGeneratingEfficiencyCurve, HydroGeneratingUnit, HydroPlantStorageKind,
    HydroPowerPlant, HydroPump, HydroPumpOpSchedule, IdentifiedObject,
    Identity, IEC61970CIMVersion, IEEE1547AbnormalPerfomanceCategory,
    IEEE1547ControlSettings, IEEE1547Info, IEEE1547IslandingCategory,
    IEEE1547NormalPerformanceCategory, IEEE1547Setting, IEEE1547TripSettings,
    Impedance, IncrementalHeatRateCurve, InflowForecast, InspectionDataSet,
    IntegerQuantity, InterrupterUnit, InterrupterUnitInfo,
    InterruptingMediumKind, IntervalBlock, IntervalReading, IOPoint,
    IrregularIntervalSchedule, IrregularTimePoint, Issuer, Jumper, Junction,
    KiloActivePower, Length, LevelVsVolumeCurve, Limit, LimitSet, Line,
    LinearShuntCompensator, LinearShuntCompensatorPhase, LineFault, LoadArea,
    LoadBreakSwitch, LoadGroup, LoadResponseCharacteristic, Location,
    LocationKind, MacroPeriodKind, Manufacturer, Mass, Measurement,
    MeasurementKind, MeasurementValue, MeasurementValueQuality,
    MeasurementValueSource, MeasuringPeriodKind, Medium, MediumKind, Meter,
    MeterMultiplier, MeterMultiplierKind, MeterReading, MeterWorkTask,
    MetrologyRequirement, Minutes, Money, MonthDay, MonthDayInterval,
    MutualCoupling, Name, NamedPhenomenon, NameType, NameTypeAuthority,
    NoLoadTest, NonConformLoad, NonConformLoadGroup, NonConformLoadSchedule,
    NonlinearShuntCompensator, NonlinearShuntCompensatorPhase,
    NonlinearShuntCompensatorPhasePoint, NonlinearShuntCompensatorPoint,
    NotificationTriggerKind, NuclearGeneratingUnit, OpenCircuitTest,
    OperatingMechanism, OperatingMechanismInfo, OperatingMechanismKind,
    OperatingParticipant, OperatingShare, OperationalLimit,
    OperationalLimitDirectionKind, OperationalLimitSet, OperationalLimitType,
    Organisation, OrganisationRole, OverfrequencyTripCurve, OverheadWireInfo,
    OvervoltageTripCurve, Ownership, ParallelLineSegment, ParentOrganization,
    PendingCalculation, PenstockLossCurve, PerCent, PerLengthImpedance,
    PerLengthLineParameter, PerLengthPhaseImpedance,
    PerLengthSequenceImpedance, Person, PersonRole, PetersenCoil,
    PetersenCoilModeKind, PhaseCode, PhaseConnectedFaultKind,
    PhaseImpedanceData, PhaseShuntConnectionKind, PhaseTapChanger,
    PhaseTapChangerAsymmetrical, PhaseTapChangerLinear,
    PhaseTapChangerNonLinear, PhaseTapChangerSymmetrical, PhaseTapChangerTable,
    PhaseTapChangerTablePoint, PhaseTapChangerTabular,
    PhenomenonClassification, PhotovoltaicUnit, Plant, PositionPoint,
    PowerCutZone, PowerElectronicsConnection, PowerElectronicsConnectionPhase,
    PowerElectronicsUnit, PowerElectronicsWindUnit, PowerSystemResource,
    PowerTransformer, PowerTransformerEnd, PowerTransformerInfo, Pressure,
    PricingStructure, Priority, Procedure, ProcedureDataSet, ProcedureKind,
    ProductAssetModel, ProtectedSwitch, PSRType, Quality61850,
    RaiseLowerCommand, RandomisationKind, RationalNumber, RatioTapChanger,
    RatioTapChangerTable, RatioTapChangerTablePoint, Reactance,
    ReactancePerLength, ReactiveCapabilityCurve, ReactivePower, Reading,
    ReadingInterharmonic, ReadingReasonKind, ReadingType, RealEnergy, Recloser,
    Register, RegularIntervalSchedule, RegularTimePoint, RegulatingCondEq,
    RegulatingControl, RegulatingControlModeKind, RegulationSchedule,
    ReportingCapability, ReportingGroup, ReportingMethodKind,
    ReportingSuperGroup, Reservoir, Resistance, ResistancePerLength,
    RightOfWay, RotatingMachine, RotationSpeed, ScaleKind, ScheduledEvent,
    ScheduledEventData, Seal, SealConditionKind, SealKind, Season,
    SeasonDayTypeSchedule, Seconds, Sectionaliser, SeriesCompensator,
    ServiceCategory, ServiceKind, ServiceLocation, ServiceMultiplier,
    ServiceMultiplierKind, SetPoint, ShortCircuitRotorKind, ShortCircuitTest,
    ShuntCompensator, ShuntCompensatorInfo, ShuntCompensatorPhase,
    ShutdownCurve, SinglePhaseKind, SmartInverterMode, SolarGeneratingUnit,
    Source, StartIgnFuelCurve, StartMainFuelCurve, StartRampCurve,
    StartupModel, StateVariable, StaticVarCompensator, StationSupply, Status,
    SteamSendoutSchedule, StreetAddress, StreetDetail, StringMeasurement,
    StringMeasurementValue, StringQuantity, Structure, StructureMaterialKind,
    StructureSupport, StructureSupportKind, SubGeographicalRegion, SubLoadArea,
    Substation, Susceptance, SusceptancePerLength, SVCControlMode,
    SvEstVoltage, SvInjection, SvPowerFlow, SvShuntCompensatorSections,
    SvStatus, SvSwitch, SvTapStep, SvVoltage, Switch, SwitchInfo,
    SwitchOperationSummary, SwitchPhase, SwitchSchedule, SynchronousMachine,
    SynchronousMachineKind, SynchronousMachineOperatingMode, TailbayLossCurve,
    TapChanger, TapChangerControl, TapChangerInfo, TapChangerTablePoint,
    TapeShieldCableInfo, TapSchedule, TargetLevelSchedule, TelephoneNumber,
    Temperature, Terminal, TestKind, ThermalGeneratingUnit,
    ThermostatController, ThermostatControlMode, TimeInterval,
    TimeIntervalKind, TopologicalIsland, TopologicalNode, TownDetail,
    TransformerApplicationKind, TransformerControlMode,
    TransformerCoreAdmittance, TransformerEnd, TransformerEndInfo,
    TransformerMeshImpedance, TransformerStarImpedance, TransformerTank,
    TransformerTankEnd, TransformerTankInfo, TransformerTest, TroubleTicket,
    UnderfrequencyTripCurve, UndervoltageTripCurve, UnitMultiplier, UnitSymbol,
    UsagePoint, UsagePointConnectedKind, UsagePointGroup, UsagePointLocation,
    UserAttribute, Validity, ValueAliasSet, ValueToAlias, Version, Voltage,
    VoltageControlZone, VoltageLevel, VoltageLimit, VoltagePerReactivePower,
    VoltVarCurve, VoltWattCurve, Volume, VolumeFlowRate, WaterLevel,
    WattVarCurve, WindGeneratingUnit, WindGenUnitKind, WindingConnection,
    WireAssemblyInfo, WireInfo, WireInsulationKind, WireMaterialKind,
    WirePhaseInfo, WirePosition, WireSpacingInfo, WireUsageKind)

__all__ = [
    'Identity',
    'ACDCTerminal' ,
    'ACLineSegment' ,
    'ACLineSegmentPhase' ,
    'AccountNotification' ,
    'AccumulationKind' ,
    'Accumulator' ,
    'AccumulatorLimit' ,
    'AccumulatorLimitSet' ,
    'AccumulatorReset' ,
    'AccumulatorValue' ,
    'ActivePowerLimit' ,
    'ActivityRecord' ,
    'AggregateKind' ,
    'AggregateScore' ,
    'AirCompressor' ,
    'AlertTypeList' ,
    'AmiBillingReadyKind' ,
    'Analog' ,
    'AnalogControl' ,
    'AnalogLimit' ,
    'AnalogLimitSet' ,
    'AnalogValue' ,
    'Analytic' ,
    'AnalyticKind' ,
    'AnalyticScore' ,
    'ApparentPowerLimit' ,
    'Appointment' ,
    'Approver' ,
    'Asset' ,
    'AssetContainer' ,
    'AssetDeployment' ,
    'AssetFunction' ,
    'AssetGroup' ,
    'AssetGroupKind' ,
    'AssetHazardKind' ,
    'AssetHealthEvent' ,
    'AssetInfo' ,
    'AssetKind' ,
    'AssetLocationHazard' ,
    'AssetModelUsageKind' ,
    'AssetOrganisationRole' ,
    'AssetOwner' ,
    'AsynchronousMachine' ,
    'AsynchronousMachineKind' ,
    'Author' ,
    'BaseFrequency' ,
    'BasePower' ,
    'BaseVoltage' ,
    'BasicIntervalSchedule' ,
    'BatteryState' ,
    'BatteryUnit' ,
    'Bay' ,
    'BranchGroup' ,
    'BranchGroupTerminal' ,
    'Breaker' ,
    'BreakerApplicationKind' ,
    'BreakerConfiguration' ,
    'BusNameMarker' ,
    'BusbarConfiguration' ,
    'BusbarSection' ,
    'BusbarSectionInfo' ,
    'Bushing' ,
    'BushingInfo' ,
    'BushingInsulationKind' ,
    'CAESPlant' ,
    'CableConstructionKind' ,
    'CableInfo' ,
    'CableOuterJacketKind' ,
    'CableShieldMaterialKind' ,
    'CatalogAssetType' ,
    'Channel' ,
    'Clamp' ,
    'Class1' ,
    'ClassificationCondition' ,
    'CogenerationPlant' ,
    'CombinedCyclePlant' ,
    'Command' ,
    'CommodityKind' ,
    'CompositeSwitch' ,
    'ConcentricNeutralCableInfo' ,
    'ConductingEquipment' ,
    'Conductor' ,
    'ConfigurationEvent' ,
    'ConformLoad' ,
    'ConformLoadGroup' ,
    'ConformLoadSchedule' ,
    'ConnectivityNode' ,
    'ConnectivityNodeContainer' ,
    'Connector' ,
    'Control' ,
    'CoolantType' ,
    'CoordinateSystem' ,
    'CorporateStandardKind' ,
    'Crew' ,
    'CrewMember' ,
    'CrewStatusKind' ,
    'CrewType' ,
    'Currency' ,
    'CurrentLimit' ,
    'Curve' ,
    'CurveData' ,
    'CurveStyle' ,
    'Customer' ,
    'CustomerAccount' ,
    'CustomerAgreement' ,
    'CustomerKind' ,
    'CustomerNotification' ,
    'Cut' ,
    'DERCurveData' ,
    'DERFunction' ,
    'DERGroupDispatch' ,
    'DERGroupForecast' ,
    'DERMonitorableParameter' ,
    'DERParameterKind' ,
    'DERUnitSymbol' ,
    'DateInterval' ,
    'DateTimeInterval' ,
    'DayType' ,
    'DecimalQuantity' ,
    'DemandResponseProgram' ,
    'DeploymentDate' ,
    'DeploymentStateKind' ,
    'Disconnector' ,
    'Discrete' ,
    'DiscreteCommand' ,
    'DiscreteValue' ,
    'DispatchSchedule' ,
    'DispatchablePowerCapability' ,
    'Document' ,
    'DuctBank' ,
    'EarthFaultCompensator' ,
    'Editor' ,
    'ElectronicAddress' ,
    'EmissionAccount' ,
    'EmissionCurve' ,
    'EmissionType' ,
    'EmissionValueSource' ,
    'EndDevice' ,
    'EndDeviceAction' ,
    'EndDeviceCapability' ,
    'EndDeviceControl' ,
    'EndDeviceControlType' ,
    'EndDeviceEvent' ,
    'EndDeviceEventDetail' ,
    'EndDeviceEventType' ,
    'EndDeviceFunction' ,
    'EndDeviceGroup' ,
    'EndDeviceInfo' ,
    'EndDeviceTiming' ,
    'EnergyArea' ,
    'EnergyConnection' ,
    'EnergyConsumer' ,
    'EnergyConsumerPhase' ,
    'EnergySource' ,
    'EnergySourcePhase' ,
    'EnvironmentalAlert' ,
    'EnvironmentalAnalog' ,
    'EnvironmentalDataAuthority' ,
    'EnvironmentalDataProvider' ,
    'EnvironmentalInformation' ,
    'EnvironmentalLocationType' ,
    'EnvironmentalPhenomenon' ,
    'EnvironmentalStringMeasurement' ,
    'Equipment' ,
    'EquipmentContainer' ,
    'EquipmentFault' ,
    'Estimate' ,
    'ExternalNetworkInjection' ,
    'FacilityKind' ,
    'Fault' ,
    'FaultCauseType' ,
    'FaultImpedance' ,
    'Feeder' ,
    'FieldDispatchHistory' ,
    'FieldDispatchStep' ,
    'FinancialInfo' ,
    'FloatQuantity' ,
    'FlowDirectionKind' ,
    'FossilFuel' ,
    'FrequencyConverter' ,
    'FuelAllocationSchedule' ,
    'FuelType' ,
    'Fuse' ,
    'GenUnitOpCostCurve' ,
    'GenUnitOpSchedule' ,
    'GeneratingUnit' ,
    'GeneratorControlMode' ,
    'GeneratorControlSource' ,
    'GeographicalRegion' ,
    'GrossToNetActivePowerCurve' ,
    'Ground' ,
    'GroundDisconnector' ,
    'GroundingImpedance' ,
    'HeatInputCurve' ,
    'HeatRateCurve' ,
    'House' ,
    'HouseCooling' ,
    'HouseHeating' ,
    'HouseThermalIntegrity' ,
    'HydroEnergyConversionKind' ,
    'HydroGeneratingEfficiencyCurve' ,
    'HydroGeneratingUnit' ,
    'HydroPlantStorageKind' ,
    'HydroPowerPlant' ,
    'HydroPump' ,
    'HydroPumpOpSchedule' ,
    'IEC61970CIMVersion' ,
    'IEEE1547AbnormalPerfomanceCategory' ,
    'IEEE1547ControlSettings' ,
    'IEEE1547Info' ,
    'IEEE1547IslandingCategory' ,
    'IEEE1547NormalPerformanceCategory' ,
    'IEEE1547Setting' ,
    'IEEE1547TripSettings' ,
    'IOPoint' ,
    'IdentifiedObject' ,
    'IncrementalHeatRateCurve' ,
    'InflowForecast' ,
    'InspectionDataSet' ,
    'IntegerQuantity' ,
    'InterrupterUnit' ,
    'InterrupterUnitInfo' ,
    'InterruptingMediumKind' ,
    'IntervalBlock' ,
    'IntervalReading' ,
    'IrregularIntervalSchedule' ,
    'IrregularTimePoint' ,
    'Issuer' ,
    'Jumper' ,
    'Junction' ,
    'LevelVsVolumeCurve' ,
    'Limit' ,
    'LimitSet' ,
    'Line' ,
    'LineFault' ,
    'LinearShuntCompensator' ,
    'LinearShuntCompensatorPhase' ,
    'LoadArea' ,
    'LoadBreakSwitch' ,
    'LoadGroup' ,
    'LoadResponseCharacteristic' ,
    'Location' ,
    'LocationKind' ,
    'MacroPeriodKind' ,
    'Manufacturer' ,
    'Measurement' ,
    'MeasurementKind' ,
    'MeasurementValue' ,
    'MeasurementValueQuality' ,
    'MeasurementValueSource' ,
    'MeasuringPeriodKind' ,
    'Medium' ,
    'MediumKind' ,
    'Meter' ,
    'MeterMultiplier' ,
    'MeterMultiplierKind' ,
    'MeterReading' ,
    'MeterWorkTask' ,
    'MetrologyRequirement' ,
    'MonthDayInterval' ,
    'MutualCoupling' ,
    'Name' ,
    'NameType' ,
    'NameTypeAuthority' ,
    'NamedPhenomenon' ,
    'NoLoadTest' ,
    'NonConformLoad' ,
    'NonConformLoadGroup' ,
    'NonConformLoadSchedule' ,
    'NonlinearShuntCompensator' ,
    'NonlinearShuntCompensatorPhase' ,
    'NonlinearShuntCompensatorPhasePoint' ,
    'NonlinearShuntCompensatorPoint' ,
    'NotificationTriggerKind' ,
    'NuclearGeneratingUnit' ,
    'OpenCircuitTest' ,
    'OperatingMechanism' ,
    'OperatingMechanismInfo' ,
    'OperatingMechanismKind' ,
    'OperatingParticipant' ,
    'OperatingShare' ,
    'OperationalLimit' ,
    'OperationalLimitDirectionKind' ,
    'OperationalLimitSet' ,
    'OperationalLimitType' ,
    'Organisation' ,
    'OrganisationRole' ,
    'OverfrequencyTripCurve' ,
    'OverheadWireInfo' ,
    'OvervoltageTripCurve' ,
    'Ownership' ,
    'PSRType' ,
    'ParallelLineSegment' ,
    'ParentOrganization' ,
    'PendingCalculation' ,
    'PenstockLossCurve' ,
    'PerLengthImpedance' ,
    'PerLengthLineParameter' ,
    'PerLengthPhaseImpedance' ,
    'PerLengthSequenceImpedance' ,
    'Person' ,
    'PersonRole' ,
    'PetersenCoil' ,
    'PetersenCoilModeKind' ,
    'PhaseCode' ,
    'PhaseConnectedFaultKind' ,
    'PhaseImpedanceData' ,
    'PhaseShuntConnectionKind' ,
    'PhaseTapChanger' ,
    'PhaseTapChangerAsymmetrical' ,
    'PhaseTapChangerLinear' ,
    'PhaseTapChangerNonLinear' ,
    'PhaseTapChangerSymmetrical' ,
    'PhaseTapChangerTable' ,
    'PhaseTapChangerTablePoint' ,
    'PhaseTapChangerTabular' ,
    'PhenomenonClassification' ,
    'PhotovoltaicUnit' ,
    'Plant' ,
    'PositionPoint' ,
    'PowerCutZone' ,
    'PowerElectronicsConnection' ,
    'PowerElectronicsConnectionPhase' ,
    'PowerElectronicsUnit' ,
    'PowerElectronicsWindUnit' ,
    'PowerSystemResource' ,
    'PowerTransformer' ,
    'PowerTransformerEnd' ,
    'PowerTransformerInfo' ,
    'PricingStructure' ,
    'Priority' ,
    'Procedure' ,
    'ProcedureDataSet' ,
    'ProcedureKind' ,
    'ProductAssetModel' ,
    'ProtectedSwitch' ,
    'Quality61850' ,
    'RaiseLowerCommand' ,
    'RandomisationKind' ,
    'RatioTapChanger' ,
    'RatioTapChangerTable' ,
    'RatioTapChangerTablePoint' ,
    'RationalNumber' ,
    'ReactiveCapabilityCurve' ,
    'Reading' ,
    'ReadingInterharmonic' ,
    'ReadingReasonKind' ,
    'ReadingType' ,
    'Recloser' ,
    'Register' ,
    'RegularIntervalSchedule' ,
    'RegularTimePoint' ,
    'RegulatingCondEq' ,
    'RegulatingControl' ,
    'RegulatingControlModeKind' ,
    'RegulationSchedule' ,
    'ReportingCapability' ,
    'ReportingGroup' ,
    'ReportingMethodKind' ,
    'ReportingSuperGroup' ,
    'Reservoir' ,
    'RightOfWay' ,
    'RotatingMachine' ,
    'SVCControlMode' ,
    'ScaleKind' ,
    'ScheduledEvent' ,
    'ScheduledEventData' ,
    'Seal' ,
    'SealConditionKind' ,
    'SealKind' ,
    'Season' ,
    'SeasonDayTypeSchedule' ,
    'Sectionaliser' ,
    'SeriesCompensator' ,
    'ServiceCategory' ,
    'ServiceKind' ,
    'ServiceLocation' ,
    'ServiceMultiplier' ,
    'ServiceMultiplierKind' ,
    'SetPoint' ,
    'ShortCircuitRotorKind' ,
    'ShortCircuitTest' ,
    'ShuntCompensator' ,
    'ShuntCompensatorInfo' ,
    'ShuntCompensatorPhase' ,
    'ShutdownCurve' ,
    'SinglePhaseKind' ,
    'SmartInverterMode' ,
    'SolarGeneratingUnit' ,
    'StartIgnFuelCurve' ,
    'StartMainFuelCurve' ,
    'StartRampCurve' ,
    'StartupModel' ,
    'StateVariable' ,
    'StaticVarCompensator' ,
    'StationSupply' ,
    'Status' ,
    'SteamSendoutSchedule' ,
    'StreetAddress' ,
    'StreetDetail' ,
    'StringMeasurement' ,
    'StringMeasurementValue' ,
    'StringQuantity' ,
    'Structure' ,
    'StructureSupport' ,
    'SubGeographicalRegion' ,
    'SubLoadArea' ,
    'Substation' ,
    'SvEstVoltage' ,
    'SvInjection' ,
    'SvPowerFlow' ,
    'SvShuntCompensatorSections' ,
    'SvStatus' ,
    'SvSwitch' ,
    'SvTapStep' ,
    'SvVoltage' ,
    'Switch' ,
    'SwitchInfo' ,
    'SwitchOperationSummary' ,
    'SwitchPhase' ,
    'SwitchSchedule' ,
    'SynchronousMachine' ,
    'SynchronousMachineKind' ,
    'SynchronousMachineOperatingMode' ,
    'TailbayLossCurve' ,
    'TapChanger' ,
    'TapChangerControl' ,
    'TapChangerInfo' ,
    'TapChangerTablePoint' ,
    'TapSchedule' ,
    'TapeShieldCableInfo' ,
    'TargetLevelSchedule' ,
    'TelephoneNumber' ,
    'Terminal' ,
    'TestKind' ,
    'ThermalGeneratingUnit' ,
    'ThermostatControlMode' ,
    'ThermostatController' ,
    'TimeInterval' ,
    'TimeIntervalKind' ,
    'TopologicalIsland' ,
    'TopologicalNode' ,
    'TownDetail' ,
    'TransformerApplicationKind' ,
    'TransformerControlMode' ,
    'TransformerCoreAdmittance' ,
    'TransformerEnd' ,
    'TransformerEndInfo' ,
    'TransformerMeshImpedance' ,
    'TransformerStarImpedance' ,
    'TransformerTank' ,
    'TransformerTankEnd' ,
    'TransformerTankInfo' ,
    'TransformerTest' ,
    'TroubleTicket' ,
    'UnderfrequencyTripCurve' ,
    'UndervoltageTripCurve' ,
    'UnitMultiplier' ,
    'UnitSymbol' ,
    'UsagePoint' ,
    'UsagePointConnectedKind' ,
    'UsagePointGroup' ,
    'UsagePointLocation' ,
    'UserAttribute' ,
    'Validity' ,
    'ValueAliasSet' ,
    'ValueToAlias' ,
    'Version' ,
    'VoltVarCurve' ,
    'VoltWattCurve' ,
    'VoltageControlZone' ,
    'VoltageLevel' ,
    'VoltageLimit' ,
    'WattVarCurve' ,
    'WindGenUnitKind' ,
    'WindGeneratingUnit' ,
    'WindingConnection' ,
    'WireAssemblyInfo' ,
    'WireInfo' ,
    'WireInsulationKind' ,
    'WireMaterialKind' ,
    'WirePhaseInfo' ,
    'WirePosition' ,
    'WireSpacingInfo' ,
    'WireUsageKind' ,
    'MonthDay' ,
    'Susceptance' ,
    'Pressure' ,
    'Hours' ,
    'Capacitance' ,
    'Voltage' ,
    'SusceptancePerLength' ,
    'Frequency' ,
    'PU' ,
    'CalculationIntervalUnitKind' ,
    'Minutes' ,
    'AngleDegrees' ,
    'ActivePowerChangeRate' ,
    'RotationSpeed' ,
    'RealEnergy' ,
    'VolumeFlowRate' ,
    'StructureMaterialKind' ,
    'CostPerHeatUnit' ,
    'Money' ,
    'Area' ,
    'AnchorKind' ,
    'Volume' ,
    'CostPerEnergyUnit' ,
    'KiloActivePower' ,
    'Displacement' ,
    'ApparentPower' ,
    'CurrentFlow' ,
    'Source' ,
    'ReactivePower' ,
    'Temperature' ,
    'Seconds' ,
    'ActivePowerPerFrequency' ,
    'Emission' ,
    'VoltagePerReactivePower' ,
    'WaterLevel' ,
    'ActivePower' ,
    'Mass' ,
    'ConductancePerLength' ,
    'Reactance' ,
    'Conductance' ,
    'ReactancePerLength' ,
    'Resistance' ,
    'CostPerVolume' ,
    'StructureSupportKind' ,
    'CostRate' ,
    'Length' ,
    'ResistancePerLength' ,
    'PerCent' ,
    'AngleRadians' ,
    'HeatRate' ,
    'Impedance'
]
